#!/usr/bin/env python

# Rnmap server v. 0.1 Beta
#
# Copyright (C) 2000 Tuomo Makinen (tmakinen@pp.htv.fi)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

"""
Rnmap server v. 0.1 Beta

Usage: rnmap_server.py [-p] [-n] [-h]

-p: port
-n: Nmap path
-h: hostname
"""

from socket import *
import os
import string
import sys
import thread
import tempfile
import time
import getopt

CRLF = "\r\n"
SUCCESFUL  = "201:" #Succesful
AUTHPERM   = "202:Access permitted"
AUTHDENI   = "401:Access denied"
SCANFAIL   = "402:Operation failed"
UNKNOWNERR = "403:Unknown command"
BADCHARS   = "404:Operation denied"

# NOTHESECHARS is list which has characters that can cause
# unwanted action, and we don't want pass them to system.
NOTHESECHARS = ["|", ";", "<", ">", "&"]
                     
Server = ''
Port = 3500
Nmapl = "/usr/bin/nmap"

try:
    opts, args = getopt.getopt(sys.argv[1:], 'p:n:h:', '--help')
except:
    print __doc__
    sys.exit()

for o, a in opts:
    if o == '-h': Server = a
    if o == '-p': Port = string.atoi(a)
    if o == '-n': Nmapl = a
    if o == '--help':
        print __doc__
        sys.exit()
    
#==========================================================================
# read configuration files ...
#==========================================================================
# filter.conf is optional file, if omitted we're not using filtering rules.
try:
    # Try to open filtering configuration
    fp = open("filter.conf")
    tmplist = fp.readlines()
    fp.close()
    clist = []
    for i in xrange(len(tmplist)):
        if "#" in tmplist[i]:
            continue
        elif tmplist[i][0:] == '\012':
            continue
        elif tmplist[i][0] == "!":
            fmethod = tmplist[i][9:16]
        else:
            clist.append(tmplist[i])
    for j in xrange(len(clist)):
      clist[j] = clist[j][:-1]
    del tmplist
    filteruse = 1
except:
    # No filtering rules used
    filteruse = None
    fmethod = None
# userlist file
try:
    fp = open("users.list")
    ul = fp.readlines()
    userlist = {}
    for i in xrange(len(ul)):
        stopsign = 0
        for j in xrange(len(ul[i])): 
            if ul[i][0] == "#":
                continue
            elif ul[i][0] == " ":
                continue
            elif ul[i][j] == ":":
                if stopsign == 0:
                    userlist[ul[i][:j]] = ul[i][j+1:-1]
                    stopsign = 1
except:
    sys.stderr.write("Error reading userlist!\a\n")
    sys.exit()
#==========================================================================

def waitconnection():
    s = socket(AF_INET, SOCK_STREAM)
    s.bind(Server, Port)
    s.listen(1)
    while 1:
        conn, addr = s.accept()
        data = conn.recv(4096)
        thread.start_new_thread(newconnection,(s, conn, addr[0], data))
        
def newconnection(s, conn, host, data):
    if filteruse != None:
        check = filter(host)
        if check == 0:
            conn.send(AUTHDENI)
            conn.close()
            wlog = time.ctime(time.time())+" Banned "+host+" host dropped.\n"
            logwriter(wlog)
    if check != 0:
        message = ploadexam(data, host)
        conn.send(message)
        conn.close()
        
    thread.exit()

def ploadexam(message, host):
    message = cleaner(message)
    cmd_point = string.rfind(message, ":")
    usr_point = string.find(message, ":")
    login = message[:usr_point]
    cmd = message[cmd_point+1:]
    passwd = message[usr_point+1:cmd_point]
    if (cmd_point != -1 and usr_point != -1) and (cmd_point != usr_point):
        if cmd == '':
            auth = authenticate(login, passwd)                        
            if auth == "true":
                wlog = time.ctime(time.time())+" User "+login+" from IP "\
                        +host+" authorized.\n"
                logwriter(wlog)
                info = AUTHPERM
            elif auth == "false":
                wlog = time.ctime(time.time())+" User "+login+" from "\
                        +host+" tried to login with bad password.\n"
                logwriter(wlog)
                info = AUTHDENI
            elif auth == "nouser":
                wlog = time.ctime(time.time())+" Unknown user from "+host\
                        +" tried to connect.\n"
                logwriter(wlog)
                info = AUTHDENI
        elif cmd != '':
            auth = authenticate(login, passwd)
            if auth == "true":
                # Check chars
                for i in xrange(len(cmd)):
                    if cmd[i] in NOTHESECHARS:
                        scanstat = 0
                        break
                    else:
                        scanstat = 1
                if scanstat == 0:    
                    wlog = time.ctime(time.time())+" User "+login+" from "+host+\
                            " used illegal parameters: "+cmd+"\n"
                    logwriter(wlog)
                    info = BADCHARS   
                elif scanstat == 1:
                    info = scanner(login, host, cmd)
            elif auth != "true":
                wlog = time.ctime(time.time())+" User "+login+" from "+host+\
                        " tried to execute scan without permission.\n"
                logwriter(wlog)
                info = AUTHDENI
    else:
        info = UNKNOWNERR
        
    return info

def scanner(login, host, cmd):
    tempfile.tempdir = "./tmp"
    tmpresult = tempfile.mktemp()
    syscmd = Nmapl+" "+cmd+" "+tmpresult+" >/dev/null 2>&1"
    scn = os.system(syscmd)
    if scn != 0:
        wlog = time.ctime(time.time())+" User "+login+" from "+host+\
               " peformed failed scan: "+cmd+"\n"
        logwriter(wlog)
        info = SCANFAIL
    else:
        wlog = time.ctime(time.time())+" User "+login+" from "+host+\
               " peformed scan: "+cmd+"\n"
        logwriter(wlog)
        try:
            fp = open(tmpresult, "r")
            info = SUCCESFUL+fp.read()
            fp.close()
            os.unlink(tmpresult)
        except:
            info = SCANFAIL
            
    return info

def authenticate(login, passwd):        
    if userlist.has_key(login):
        if userlist[login] == passwd:
            authentication = "true"
        else:
            authentication = "false"
    else:
        authentication = "nouser"

    return (authentication)
        
def logwriter(wlog):
    try:
        fp = open("rnmap_server.log", "a")
        fp.write(wlog)
        fp.close()
    except:
        wlog = time.ctime(time.time())+" Can't write to disk!\n"
        logwrite(wlog)

def cleaner(message):
    if len(message) >= 2:
        if message[-2:] == CRLF:
            message = message[:-2]
        elif message[-1:] in CRLF:
            message = message[:-1]
    
    return message

def filter(host):
    check = None
    hcheck = host
    if fmethod == "drpexcp":
        if hcheck in clist:
            check = 1
        else:
            check = 0
    if fmethod == "allexcp":
        if hcheck in clist:
            check = 0
        else:
            check = 1
            
    return check

# Fork to background ...
pid = os.fork()
if pid != 0:
    sys.exit()
else:
    waitconnection()
